/*
 * @(#)MaachangSyncProtocol.java
 *
 * Copyright (c) 2007 masahito suzuki, Inc. All Rights Reserved
 */
package org.maachang.sync ;

import org.maachang.util.ConvertBinary;

/**
 * プロトコル定義.
 *
 * @version 2007/05/19
 * @author  masahito suzuki
 * @since   MaachangSync 1.00
 */
public class MaachangSyncProtocol {
    
    /**
     * プロトコルヘッダ : 最終更新日.
     */
    public static final byte[] HD_LAST_UPATE = {
        ( byte )0x3a,( byte )0xfa
    } ;
    
    /**
     * プロトコルヘッダ長 : 最終更新日.
     */
    public static final int HD_LAST_UPDATE_LENGTH = 2 + 32 + 1 + 4 + 8 + 4 ;
    // プロトコル長.
    // 最終更新日.
    // ターゲットディレクトリ名長.
    
    /**
     * プロトコルヘッダ : 同期データ開始情報.
     */
    public static final byte[] HD_SYNC_START_FILE = {
        ( byte )0x3a,( byte )0x1f
    } ;
    
    /**
     * プロトコルヘッダ長 : 同期データ開始情報.
     */
    public static final int HD_SYNC_START_FILE_LENGTH = 2 + 32 + 1 + 4 + 4 + 8 + 4 ;
    // プロトコル長.
    // ファイル項番.
    // 全ファイルサイズ.
    // パス名長.
    
    /**
     * プロトコルヘッダ : 同期データ情報.
     */
    public static final byte[] HD_SYNC_FILE = {
        ( byte )0x3a,( byte )0x5c
    } ;
    
    /**
     * プロトコルヘッダ長 : 同期データ情報.
     */
    public static final int HD_SYNC_FILE_LENGTH = 2 + 32 + 1 + 4 + 4 + 4 + 4 ;
    // プロトコル長.
    // ファイル項番.
    // シーケンスID.
    // １回のファイルサイズ.
    
    /**
     * プロトコルヘッダ : 終了情報.
     */
    public static final byte[] HD_END = {
        ( byte )0x3a,( byte )0xed
    } ;
    
    /**
     * プロトコルヘッダ長 : 終了情報.
     */
    public static final int HD_END_LENGTH = 2 + 32 + 1 + 4 + 4 ;
    // プロトコル長.
    // 送信されたファイル数.
    
    /**
     * データ領域ポイント.
     */
    public static final int DATA_POINT = 2 + 32 + 1 + 4 ;
    // ヘッダ.
    // 暗号キー.
    // チェックコード.
    // プロトコル長.
    
    /**
     * 暗号チェックコード位置.
     */
    public static final int CHECK_POINT = 2 + 32 ;
    
    /**
     * ヘッダ情報を取得.
     * <BR><BR>
     * ヘッダ情報を取得します.
     * <BR>
     * @param buf 対象の受信バッファを取得します.
     * @return byte[] ヘッダ情報が返されます.
     */
    public static final byte[] getHeader( byte[] buf ) {
        byte[] ret = new byte[ 2 ] ;
        System.arraycopy( buf,0,ret,0,2 ) ;
        return ret ;
    }
    
    /**
     * 暗号キー情報を取得.
     * <BR><BR>
     * 暗号キー情報を取得します.
     * <BR>
     * @param buf 対象の受信バッファを取得します.
     * @return byte[] 暗号キー情報が返されます.
     */
    public static final byte[] getKey( byte[] buf ) {
        byte[] ret = new byte[ 32 ] ;
        System.arraycopy( buf,2,ret,0,32 ) ;
        return ret ;
    }
    
    /**
     * 暗号コードを取得.
     * <BR><BR>
     * 暗号コードを取得します.
     * <BR>
     * @param buf 対象の受信バッファを取得します.
     * @return int 暗号コード情報が返されます.
     */
    public static final int getCode( byte[] buf ) {
        return ( ( int )buf[ CHECK_POINT ] ) & 0x000000ff ;
    }
    
    /**
     * 暗号コードを取得.
     * <BR><BR>
     * 暗号コードを取得します.
     * <BR>
     * @param buf 対象の受信バッファを取得します.
     * @return int 暗号コード情報が返されます.
     * @exception Exception 例外.
     */
    public static final int getProtocolLength( byte[] buf )
        throws Exception {
        return ConvertBinary.convertInt( CHECK_POINT+1,buf ) ;
    }
    
}
