/*
 * @(#)ByteUtil.java
 *
 * Copyright (c) 2007 masahito suzuki, Inc. All Rights Reserved
 */
package org.maachang.util ;

/**
 * バイナリ管理オブジェクト.
 *
 * @version 2007/05/19
 * @author  masahito suzuki
 * @since   MaachangSync 1.00
 */
public class ByteUtil {
    
    /**
     * バイナリ情報.
     */
    private byte[] binary = null ;
    
    /**
     * 現在バイナリサイズ.
     */
    private int length = 0 ;
    
    /**
     * コンストラクタ.
     */
    public ByteUtil() {
        
    }
    
    /**
     * クリア.
     * <BR><BR>
     * オブジェクトをクリアします.
     */
    public void clear() {
        binary = null ;
        length = 0 ;
    }
    
    /**
     * 情報追加.
     * <BR><BR>
     * 情報を追加します.
     * <BR>
     * @param binary 追加対象のバイナリを設定します.
     * @param offset 追加対象のオフセット値を設定します.
     * @param length 追加対象の情報長を設定します.
     */
    public void add( byte[] binary,int offset,int length ) {
        int len ;
        if( binary == null || ( len = binary.length ) <= 0 || offset >= len ) {
            return ;
        }
        
        if( length <= 0 ) {
            length = len ;
        }
        if( ( offset + length ) >= len ) {
            length = len - offset ;
        }
        
        if( this.binary == null || this.binary.length <= 0 ) {
            this.binary = new byte[ length ] ;
            this.length = length ;
            System.arraycopy( binary,offset,this.binary,0,length ) ;
        }
        else {
            byte[] tmp = new byte[ this.length + length ] ;
            System.arraycopy( this.binary,0,tmp,0,this.length ) ;
            System.arraycopy( binary,offset,tmp,this.length,length ) ;
            this.binary = tmp ;
            this.length = this.binary.length ;
        }
    }
    
    /**
     * 情報削除.
     * <BR><BR>
     * 情報を削除します.
     * <BR>
     * @param length 削除対象の情報長を設定します.
     */
    public void remove( int length ) {
        if( length >= this.length ) {
            this.binary = null ;
            this.length = 0 ;
        }
        else {
            int len = this.length - length ;
            byte[] tmp = new byte[ len ] ;
            System.arraycopy( this.binary,length,tmp,0,len ) ;
            this.binary = tmp ;
            this.length = len ;
        }
    }
    
    /**
     * 情報取得.
     * <BR><BR>
     * 情報を取得します.
     * <BR>
     * @param out 格納対象のバイナリ情報を設定します.
     * @return int 取得された情報長が返されます.
     */
    public int get( byte[] out ) {
        int len ;
        if( out == null || ( len = out.length ) <= 0 ) {
            return -1 ;
        }
        
        if( this.binary == null || this.length <= 0 ) {
            return -1 ;
        }
        if( len >= this.length ) {
            len = this.length ;
        }
        
        System.arraycopy( this.binary,0,out,0,len ) ;
        return len ;
    }
    
    /**
     * 情報サイズを取得.
     * <BR><BR>
     * 情報サイズを取得します.
     * <BR>
     * @return int 情報サイズが返されます.
     */
    public int size() {
        return this.length ;
    }
}
